"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _interopRequireWildcard2 = _interopRequireDefault(require("@babel/runtime/helpers/interopRequireWildcard"));

var _typescript = require("typescript");

var _plugin = _interopRequireDefault(require("@start/plugin"));

var _default = (outDirRelative, userOptions) => (0, _plugin.default)('typescriptGenerate', ({
  logPath
}) => async ({
  files
}) => {
  const {
    createProgram
  } = await Promise.resolve().then(() => (0, _interopRequireWildcard2.default)(require('typescript')));
  const path = await Promise.resolve().then(() => (0, _interopRequireWildcard2.default)(require('path')));
  const options = {
    allowSyntheticDefaultImports: true,
    esModuleInterop: true,
    moduleResolution: _typescript.ModuleResolutionKind.NodeJs,
    target: _typescript.ScriptTarget.ESNext,
    module: _typescript.ModuleKind.ESNext,
    ...userOptions,
    declarationDir: path.resolve(outDirRelative),
    declaration: true,
    emitDeclarationOnly: true
  };
  const filePaths = files.map(file => file.path);
  filePaths.forEach(logPath);
  const program = createProgram(filePaths, options);
  const emitResult = program.emit();
  const allDiagnostics = (0, _typescript.getPreEmitDiagnostics)(program).concat(emitResult.diagnostics);
  allDiagnostics.forEach(diagnostic => {
    if (diagnostic.file) {
      const {
        line,
        character
      } = diagnostic.file.getLineAndCharacterOfPosition(diagnostic.start);
      const message = (0, _typescript.flattenDiagnosticMessageText)(diagnostic.messageText, '\n');
      console.log(`${diagnostic.file.fileName} (${line + 1},${character + 1}): ${message}`);
    } else {
      console.log(`${(0, _typescript.flattenDiagnosticMessageText)(diagnostic.messageText, '\n')}`);
    }
  });
  return {
    files
  };
});

exports.default = _default;